
#define TWITIMEOUT 250 //time in microseconds for communication timeout
byte answer = 0;

void resetTWI(void) //resets TWI pins and registers, in case of accidents
{
  // release bus
  TWCR = _BV(TWEN) | _BV(TWIE) | _BV(TWEA) | _BV(TWINT);
  delayMicroseconds(4);
  TWCR &= ~(1<<TWEN); //disable TWI
  //release TWI lines:
  pinMode(18, INPUT);
  pinMode(19, INPUT);
  // digitalWrite(18, HIGH); //activat internal pullups
  // digitalWrite(19, HIGH);
  //delay(1); //wait for the lines to be released
  //Wire.begin(); //TWI reinitialisieren, hier wird TWI state auf ready gesetzt
  init_twi();
  // TWBR = ((CPU_FREQ / TWI_FREQ) - 16) / 2; //set TWI frequency
  //  Serial.println("reset TWI");
  twireset++;
}
void init_twi(void) //copied from arduino library
{
  // initialize state


#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega8__) || defined(__AVR_ATmega328P__)
  // activate internal pull-ups for twi
  // as per note from atmega8 manual pg167
  sbi(PORTC, 4);
  sbi(PORTC, 5);
#else
  // activate internal pull-ups for twi
  // as per note from atmega128 manual pg204
  sbi(PORTD, 0);
  sbi(PORTD, 1);
#endif

  // initialize twi prescaler and bit rate
  cbi(TWSR, TWPS0);
  cbi(TWSR, TWPS1);
  TWBR = ((CPU_FREQ / TWI_FREQ) - 16) / 2;

  /* twi bit rate formula from atmega128 manual pg 204
   SCL Frequency = CPU Clock Frequency / (16 + (2 * TWBR))
   note:  should be 10 or higher for master mode
   It is 72 for a 16mhz Wiring board with 100kHz TWI */

  // enable twi module, acks, and twi interrupt
  TWCR = _BV(TWEN) | _BV(TWIE) | _BV(TWEA);

}





byte TWI_initsend(byte address)
{
 // delay(1);
  byte i = 0;
  TWCR = (1<<TWINT)|(1<<TWSTA)| (1<<TWEN);
 // Serial.println("sending start condition");
  while (!(TWCR & (1<<TWINT))) 
  {
    wait();
    if(i > TWITIMEOUT) 
    {
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA); //release bus 
      return 1;
    }
    i++;
  }
  switch (TWSR & 0xF8) //check status register
  {
  case TW_START: //ok
    break;
  case TW_REP_START: //auch ok
    break;
  default:
    return TWSR & 0xF8;
  } 
  TWDR = (address)<<1; //shift damit bit0 = 0 ist, heisst write command 
  TWCR = (1<<TWINT) | (1<<TWEN);
 // Serial.println("sending address");
  i = 0;
  while (!(TWCR & (1<<TWINT))) 
  {
    wait();
    if(i > TWITIMEOUT) 
    {
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA); //release bus 
      return 255;
    }
    i++;
  }
  switch (TWSR & 0xF8)
  {
  case TW_MT_SLA_ACK: //ok
    break;
  default:
    return TWSR & 0xF8;
  } 
  return 0;
}

byte TWI_sendbyte(byte data) //sends out one byte to current slave (use initsend to initialize communication)
{
  // Serial.print("sending data: ");
  //Serial.println(data,HEX);
  byte i = 0;
  TWDR = data;
  TWCR = (1<<TWINT) | (1<<TWEN);
  while (!(TWCR & (1<<TWINT))) 
  {
    wait();
    if(i > TWITIMEOUT) 
    {
      //Serial.print("timeout");
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus
      return 255;
    }
    i++;
  }
  switch (TWSR & 0xF8)
  {
  case TW_MT_DATA_ACK: //ok
    break;
  default:
    return TWSR & 0xF8;
  } 
  return 0;
}


byte TWI_initreceive(byte address)
{
  //delay(10);
// Serial.println("setting to receive data");
  byte i = 0;
  TWCR = (1<<TWINT)|(1<<TWSTA)| (1<<TWEN); //send (repeat) start
  while (!(TWCR & (1<<TWINT))) 
  {
    wait();
    if(i > TWITIMEOUT) 
    {
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA); //release bus 
      return 255;
    }
    i++;
  }
  switch (TWSR & 0xF8)
  {
  case TW_START: //ok
    break;
  case TW_REP_START: //auch ok
    break;
  default:
    return TWSR & 0xF8;
  } 
//  Serial.println("sending address");
  TWDR = (address)<<1 | 0x01; //shift und bit0 = 1, heisst read command 
  TWCR = (1<<TWINT) | (1<<TWEN);
  i = 0;
  while (!(TWCR & (1<<TWINT))) 
  {
    wait();
    if(i > TWITIMEOUT)
    {
      //Serial.println("timeout sending address");
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus
      return 255;
    }
    i++;
  }
  switch (TWSR & 0xF8)
  {
  case TW_MR_SLA_ACK: //ok
    break;
  default:
    return TWSR & 0xF8;
  }
  return 0;
}


byte TWI_receivebyte(void)//byte *answer) //receive single byte (bus is released afterwards)
{
 // Serial.println("receiving byte");
  byte i = 0;
  TWCR = (1<<TWINT) | (1<<TWEN);  //jetzt empfange byte, aber kein ACK, d.h. nur ein byte (TWEA müsste sonst auch noch 1 sein, dann kommt danach noch ein byte)
  i = 0;
  while (!(TWCR & (1<<TWINT)))  //warte auf ende der transmission
  {
    wait();
    if(i > TWITIMEOUT) 
    {
     // Serial.println("timeout");
      TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus
      return -1;
    }
    i++;
  }
  // Serial.println("byte received: ");
  answer = TWDR; //daten vom slave sind jetzt im buffer
 // Serial.println(TWDR,HEX);
  return 0;
}


int sendcolors(byte lamp)
{
 // Serial.print("sending to lamp ");
 // Serial.println(lamp, DEC);

  byte error;
  do
  {
    error = TWI_initsend(16+lamp);
  }
  while(error);
  //	TW_MT_DATA_NACK -> nack während send

  error = TWI_sendbyte(color[RED][lamp]);
  if(error) 
  {
    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
    return error;
  }
  error = TWI_sendbyte(color[GREEN][lamp]);
  if(error) 
  {
    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
    return error;
  }
  error = TWI_sendbyte(color[BLUE][lamp]);
  if(error) 
  {
    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
    return error;
  }
  uint16_t checksum = calc_CRC16(lamp); 
  error = TWI_sendbyte((uint8_t)(checksum>>8));
  if(error) 
  {
    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
    return error;
  }
  error = TWI_sendbyte((uint8_t)(checksum));
  if(error) 
  {
    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
    return error;
  }
  TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus

  delayMicroseconds(16);//wait for slave to process data
  //byte answer = 0;

  //  for(byte j = 0; j< 7; j++)
  //  {
  do
  {
    error = TWI_initreceive(16+lamp);
    // Serial.println(error,HEX);
    if(error==0) error = TWI_receivebyte();//&answer);
  } 
  while(error);
  if(answer == 0xFF) return 0; //slave sends 0xff if all is ok
  else
  {
    byte answercheck = 0;
    //now count number of "1" in the answer
    for(int j = 0; j<8; j++)
    {
      answercheck += answer&0x01; 
      answer = answer>>1;
    }
    if(answercheck >= 4) //if at least 4 ones in answer (no transmission error: answer = 0xFF)
    {

      TWIwatchdog = 0; //reset watchdog if successful
      return 0;
    }
    else {
      TWIwatchdog = 0; //reset watchdog 
      // Serial.println("transmission error ");
      return answer;
    } 

  }
  // }
  //Serial.println(answer,HEX);

  //  if(answer == checksum)
  //  {
  //    TWIwatchdog = 0; //reset watchdog if successful
  //  }
  //  else {
  //    TWIwatchdog = 0; //reset watchdog 
  //    return 2;
  //  }

  //now send go signal (=0x55)
  //  do
  //  {
  //    error = TWI_initsend(16+lamp);
  //  }
  //  while(error);
  //  error = TWI_sendbyte(0x55);
  //  if(error) 
  //  {
  //    TWCR = (1<<TWINT)|(1<<TWEN)| (1<<TWEA)| (1<<TWSTO); //send stop, release bus 
  //    return error;
  //  }
  // 

  return 0;
}





