

#define RESOLUTION 13 //defines resolution of output in bits
//note: with a prescaler of 8, maximum flicker free resolution is 13bit (244Hz @ 16MHz CPU clock)

ISR(TIMER2_OVF_vect) //timer2 overflow
{

  if(timer2counter==0)
  {
    //switch off all outputs
    TCCR1A &= ~(1 << COM1A1); 
    TCCR1A &= ~(1 << COM1B1); 
    TCCR2A &= ~(1 << COM2A1);

    TCCR1B = 0; //stop timer
    TCCR2B = 0; //stop timer




    //set pwm periods for first timer cycle if necessary
    if(coloroverflowvalue[0] == 0) OCR1A = colorcomparevalue[0]; 
    else OCR1A = 255;
    if(coloroverflowvalue[1] == 0) OCR1B = colorcomparevalue[1];
    else OCR1B = 255;
    if(coloroverflowvalue[2] == 0) OCR2A = colorcomparevalue[2];
    else OCR2A = 255;

    //synchronize timers
    TCNT2 = 255;
    TCNT1 = 255; //timer1 is activated just after timer2
    
    if(pwmbufferready) //if new color is ready, copy buffer
    {
      for(uint8_t i = 0; i < 3; i++)
      {
        coloroverflowvalue[i] = coloroverflowvalue_buffer[i];
        colorcomparevalue[i] = colorcomparevalue_buffer[i];
      }
      pwmbufferready = 0;
    }


    TCCR1A |= (1 << COM1A1);  //enable pwm output A1 (pin 9)
    TCCR1A |= (1 << COM1B1);  //enable pwm output B1 (pin 10)
    TCCR2A |= (1 << COM2A1);  //enable pwm output A2 (pin 11)

    //start timers
    TCCR2B = (0 << WGM22) | (0 << CS22) | (1 << CS21) | (0 << CS20);  //prescaler 8
    TCCR1B = (0 << WGM13) | (1 << WGM12) | (0 << CS12) | (1 << CS11) | (0 << CS10); //prescaler 8, fast pwm 8 bit

    while(TCNT2 > 250) //wait for timer2 to overflow
    {
      asm volatile ("nop"); //wait 
    }
    
    //both timers need to overflow to update the OCRnx values!

    TIFR2 = 0x01; //clear timer overflow interrupt flag

    if(TWIwatchdog > 16) //=kein TWI signal für mehr als x ticks = 50/244Hz = 20ms
    {
      TWIwatchdog = 0;
      resetTWI();
    } 
    
    TWIwatchdog++;


  }


  //switch off output compare unit if dutycycle is reached
  if(OCR1A == 0)   TCCR1A &= ~(1 << COM1A1); 
  if(OCR1B == 0)   TCCR1A &= ~(1 << COM1B1); 
  if(OCR2A == 0)   TCCR2A &= ~(1 << COM2A1); 

  //set compare value to zero if this is last cycle 
  if(coloroverflowvalue[0] == timer2counter) OCR1A = 0;  
  if(coloroverflowvalue[1] == timer2counter) OCR1B = 0; 
  if(coloroverflowvalue[2] == timer2counter) OCR2A = 0; 


  timer2counter++; //increse period counter, overflows at 255->0
  if(timer2counter == (uint16_t)(1<<(RESOLUTION-8))-1) timer2counter = 0;

//now, the next timer period is evaluated:

    //set compare values, are set to 0 in next interrupt (OCR values are only updated when timer overflows!)
  if(coloroverflowvalue[0] == timer2counter) OCR1A = colorcomparevalue[0] ;
  if(coloroverflowvalue[1] == timer2counter) OCR1B = colorcomparevalue[1] ;
  if(coloroverflowvalue[2] == timer2counter) OCR2A = colorcomparevalue[2] ;
}





void setsoftpwm() //sets compare matches and overflow values of softpwm
{
  pwmbufferready = 0; //pwm buffer is now changing
  
   coloroverflowvalue_buffer[0] = pgm_read_word(&pwmtable_13[color[RED][0]])/256; 
   coloroverflowvalue_buffer[1] = pgm_read_word(&pwmtable_13[color[GREEN][0]])/256;
   coloroverflowvalue_buffer[2] = pgm_read_word(&pwmtable_13[color[BLUE][0]])/256;
  
   colorcomparevalue_buffer[0] =  pgm_read_word(&pwmtable_13[color[RED][0]]) - (coloroverflowvalue[0]<<8);
   colorcomparevalue_buffer[1] =  pgm_read_word(&pwmtable_13[color[GREEN][0]])  - (coloroverflowvalue[1]<<8);
   colorcomparevalue_buffer[2] =  pgm_read_word(&pwmtable_13[color[BLUE][0]])  - (coloroverflowvalue[2]<<8);


  static unsigned long factor = (unsigned long)255*(unsigned long)255*(unsigned long)255/((long)(1<<RESOLUTION) - 1);

  
 // coloroverflowvalue_buffer[0] = 0;//((uint32_t)color[RED][0]*(uint32_t)color[RED][0]*(uint32_t)color[RED][0])/factor/256;  // kubisch zu x-bit overflow
 // coloroverflowvalue_buffer[1] = ((uint32_t)color[GREEN][0]*(uint32_t)color[GREEN][0]*(uint32_t)color[GREEN][0])/factor/256; 
 // coloroverflowvalue_buffer[2]  = ((uint32_t)color[BLUE][0]*(uint32_t)color[BLUE][0]*(uint32_t)color[BLUE][0])/factor/256; 

  
 // colorcomparevalue_buffer[0] = 150;//((uint32_t)color[RED][0]*(uint32_t)color[RED][0]*(uint32_t)color[RED][0])/factor - (coloroverflowvalue[0]*256);
//  colorcomparevalue_buffer[1] =  ((uint32_t)color[GREEN][0]*(uint32_t)color[GREEN][0]*(uint32_t)color[GREEN][0])/factor - (coloroverflowvalue[1]*256);
 // colorcomparevalue_buffer[2] =  ((uint32_t)color[BLUE][0]*(uint32_t)color[BLUE][0]*(uint32_t)color[BLUE][0])/factor - (coloroverflowvalue[2]*256);

  pwmbufferready = 1;
}














