#define PRESCALER 8 //timer1 prescaler


ISR(TIMER1_OVF_vect) //timer1 runs witith prescaler of 8 in 8bit mode
{
//as timer1 is also used with the input capture unit for the IR receiver, 
//longer time intervals must be able to be captured (i.e. more than 30ms for a full IR command)
//total time is counted here

time+= 256; //add to time



}


//INPUT CAPTURE INTERRUPT, used to detect pulse signal from IR receiver



ISR(TIMER1_CAPT_vect){

  static uint8_t repeat_init = 0; 
  static uint32_t lasticr = 0; //timer value at last capture
  uint32_t duration;


  cli(); //disable interrupts (=do not chasenge "time" during this operation)

  uint32_t icr = ICR1 + time; //read time of interrupt
  
  sei();
  
     duration = icr - lasticr; //time since last interrupt, is always correct because unsigned numbers overflow at 0
    lasticr = icr;

 
  if(duration > 2000000/PRESCALER) //invalid pulse length = noise or initial pulse
  {
    commandindex = 0;
    repeat_init = 0;
   // Serial.println("IR ovf");
  }
  else if(duration > 190000/PRESCALER && duration < 240000/PRESCALER) //if init pulse (ca. 850 @ prescaler 512)
  {
    repeat_init = 0;    
   // lasticr = TCNT1;
    commandindex =0;
    //Serial.println("IR init");
   
  }
  else if(duration > 520000/PRESCALER && duration < 740000/PRESCALER && (commandindex == 32)) //if repeat init pulse command, usually 2500 ticks long@ prescaler 512, and a command was just received 
  {
    repeat_init = 1; //this is the init pulse of the repeat command
     // Serial.println("IR rep init");	
  }
  else if(duration > 140000/PRESCALER && duration < 200000/PRESCALER && (repeat_init==1)) //is this the short repeat pulse? -> this is a repeat command, 
  {
      //    Serial.println("IR rep short");	
     repeat_init = 2; //repeat successful
     command_repeat++; //repeat the last command that was received 	
     commandrepeattiming = millis(); //signal main loop when command was received (used to reset repeat if not pressed anymore), time between repeats is 110ms
  }
  else if (duration > 1200000/PRESCALER && duration < 1600000/PRESCALER &&  repeat_init == 2) //if long repeat pulse
  {
       //   Serial.println("IR rep long");	
     repeat_init = 1;
  }
  else  //if pulse is not command pulse, start counting the bits (first 16 "address" bits are sent (value: 255), then 16 command bits are sent)
  {
    repeat_init = 0;
    commandindex ++; 
  }

  if(commandindex>15 && commandindex <33)//limit command to 16bit
  {
    if(repeat_init == 0)
    {
    if(duration < 24000/PRESCALER) 
    {
      command &= ~(1 << (commandindex- 16)); //LSB is sent first, so shift increases with bit number
      //Serial.print(".");
    }
    else
    {
        //Serial.print("-"); 
      command |= (1 << (commandindex - 16)); //LSB is sent first, so shift increases with bit number
    }

    if(commandindex == 32) 
    {
          //   Serial.println(" ");  
      command_received = 1;
    }
    }
  }
  

}

/*
example durations of one button click and hold @ prescaler 512:
18408  -> initial long init pulse

note: long init pulses are not always the same! but short one is, so go for that one!

845 -> short init pulse

71 -> first bit = 0

71

69 ->0

71

71

69

71

72

140  ->1

140

142

140

140

142

140

141

142

69

71

140

71

71

69

71

71

140

141

71

140

142

140

140

2534  -> repeat init pulse

705

6053

705

6052

705

6053

705
*/


